@@ -6,7 +6,7 @@
 -- Maintainer  : Roman Leshchinskiy <rl@cse.unsw.edu.au>
 -- Stability   : experimental
 -- Portability : portable
--- 
+--
 -- Fusion-related utility types
 --
 
@@ -16,12 +16,18 @@ module Data.Vector.Fusion.Util (
   delay_inline, delayed_min
 ) where
 
+import Control.Applicative (Applicative(..))
+
 -- | Identity monad
 newtype Id a = Id { unId :: a }
 
 instance Functor Id where
   fmap f (Id x) = Id (f x)
 
+instance Applicative Id where
+  pure = Id
+  Id f <*> Id x = Id (f x)
+
 instance Monad Id where
   return     = Id
   Id x >>= f = f x
@@ -32,6 +38,10 @@ data Box a = Box { unBox :: a }
 instance Functor Box where
   fmap f (Box x) = Box (f x)
 
+instance Applicative Box where
+  pure = Box
+  Box f <*> Box x = Box (f x)
+
 instance Monad Box where
   return      = Box
   Box x >>= f = f x
