@@ -1,4 +1,4 @@
-{-# LANGUAGE MultiParamTypeClasses, BangPatterns, ScopedTypeVariables #-}
+{-# LANGUAGE CPP, MultiParamTypeClasses, FlexibleContexts, BangPatterns, TypeFamilies, ScopedTypeVariables #-}
 -- |
 -- Module      : Data.Vector.Generic.Mutable
 -- Copyright   : (c) Roman Leshchinskiy 2008-2010
@@ -7,7 +7,7 @@
 -- Maintainer  : Roman Leshchinskiy <rl@cse.unsw.edu.au>
 -- Stability   : experimental
 -- Portability : non-portable
--- 
+--
 -- Generic interface to mutable vectors
 --
 
@@ -34,13 +34,14 @@ module Data.Vector.Generic.Mutable (
 
   -- ** Growing
   grow, unsafeGrow,
+  growFront, unsafeGrowFront,
 
   -- ** Restricting memory usage
   clear,
 
   -- * Accessing individual elements
-  read, write, swap,
-  unsafeRead, unsafeWrite, unsafeSwap,
+  read, write, modify, swap, exchange,
+  unsafeRead, unsafeWrite, unsafeModify, unsafeSwap, unsafeExchange,
 
   -- * Modifying vectors
 
@@ -49,18 +50,23 @@ module Data.Vector.Generic.Mutable (
 
   -- * Internal operations
   mstream, mstreamR,
-  unstream, unstreamR,
+  unstream, unstreamR, vunstream,
   munstream, munstreamR,
   transform, transformR,
   fill, fillR,
   unsafeAccum, accum, unsafeUpdate, update, reverse,
-  unstablePartition, unstablePartitionStream, partitionStream
+  unstablePartition, unstablePartitionBundle, partitionBundle
 ) where
 
-import qualified Data.Vector.Fusion.Stream      as Stream
-import           Data.Vector.Fusion.Stream      ( Stream, MStream )
-import qualified Data.Vector.Fusion.Stream.Monadic as MStream
-import           Data.Vector.Fusion.Stream.Size
+import           Data.Vector.Generic.Mutable.Base
+import qualified Data.Vector.Generic.Base as V
+
+import qualified Data.Vector.Fusion.Bundle      as Bundle
+import           Data.Vector.Fusion.Bundle      ( Bundle, MBundle, Chunk(..) )
+import qualified Data.Vector.Fusion.Bundle.Monadic as MBundle
+import           Data.Vector.Fusion.Stream.Monadic ( Stream )
+import qualified Data.Vector.Fusion.Stream.Monadic as Stream
+import           Data.Vector.Fusion.Bundle.Size
 import           Data.Vector.Fusion.Util        ( delay_inline )
 
 import Control.Monad.Primitive ( PrimMonad, PrimState )
@@ -70,23 +76,12 @@ import Prelude hiding ( length, null, replicate, reverse, map, read,
 
 #include "vector.h"
 
+{-
+type family Immutable (v :: * -> * -> *) :: * -> *
+
 -- | Class of mutable vectors parametrised with a primitive state token.
 --
--- Minimum complete implementation:
---
---   * 'basicLength'
---
---   * 'basicUnsafeSlice'
---
---   * 'basicOverlaps'
---
---   * 'basicUnsafeNew'
---
---   * 'basicUnsafeRead'
---
---   * 'basicUnsafeWrite'
---
-class MVector v a where
+class MBundle.Pointer u a => MVector v a where
   -- | Length of the mutable vector. This method should not be
   -- called directly, use 'length' instead.
   basicLength       :: v s a -> Int
@@ -128,6 +123,10 @@ class MVector v a where
   -- not be called directly, use 'set' instead.
   basicSet         :: PrimMonad m => v (PrimState m) a -> a -> m ()
 
+  basicUnsafeCopyPointer :: PrimMonad m => v (PrimState m) a
+                                        -> Immutable v a
+                                        -> m ()
+
   -- | Copy a vector. The two vectors may not overlap. This method should not
   -- be called directly, use 'unsafeCopy' instead.
   basicUnsafeCopy  :: PrimMonad m => v (PrimState m) a   -- ^ target
@@ -170,6 +169,14 @@ class MVector v a where
                | otherwise = basicUnsafeCopy (basicUnsafeSlice i (n-i) v)
                                              (basicUnsafeSlice 0 (n-i) v)
 
+  {-# INLINE basicUnsafeCopyPointer #-}
+  basicUnsafeCopyPointer !dst !src = do_copy 0 src
+    where
+      do_copy !i p | Just (x,q) <- MBundle.pget p = do
+                                                      basicUnsafeWrite dst i x
+                                                      do_copy (i+1) q
+                   | otherwise = return ()
+
   {-# INLINE basicUnsafeCopy #-}
   basicUnsafeCopy !dst !src = do_copy 0
     where
@@ -180,7 +187,7 @@ class MVector v a where
                             basicUnsafeWrite dst i x
                             do_copy (i+1)
                 | otherwise = return ()
-  
+
   {-# INLINE basicUnsafeMove #-}
   basicUnsafeMove !dst !src
     | basicOverlaps dst src = do
@@ -196,6 +203,7 @@ class MVector v a where
         return v'
     where
       n = basicLength v
+-}
 
 -- ------------------
 -- Internal functions
@@ -227,15 +235,15 @@ unsafePrepend1 v i x
                   unsafeWrite v i' x
                   return (v, i')
   | otherwise = do
-                  (v', i) <- enlargeFront v
-                  let i' = i-1
+                  (v', j) <- enlargeFront v
+                  let i' = j-1
                   INTERNAL_CHECK(checkIndex) "unsafePrepend1" i' (length v')
                     $ unsafeWrite v' i' x
                   return (v', i')
 
-mstream :: (PrimMonad m, MVector v a) => v (PrimState m) a -> MStream m a
+mstream :: (PrimMonad m, MVector v a) => v (PrimState m) a -> Stream m a
 {-# INLINE mstream #-}
-mstream v = v `seq` n `seq` (MStream.unfoldrM get 0 `MStream.sized` Exact n)
+mstream v = v `seq` n `seq` (Stream.unfoldrM get 0)
   where
     n = length v
 
@@ -245,10 +253,10 @@ mstream v = v `seq` n `seq` (MStream.unfoldrM get 0 `MStream.sized` Exact n)
           | otherwise = return $ Nothing
 
 fill :: (PrimMonad m, MVector v a)
-           => v (PrimState m) a -> MStream m a -> m (v (PrimState m) a)
+     => v (PrimState m) a -> Stream m a -> m (v (PrimState m) a)
 {-# INLINE fill #-}
 fill v s = v `seq` do
-                     n' <- MStream.foldM put 0 s
+                     n' <- Stream.foldM put 0 s
                      return $ unsafeSlice 0 n' v
   where
     {-# INLINE_INNER put #-}
@@ -257,14 +265,15 @@ fill v s = v `seq` do
                   $ unsafeWrite v i x
                 return (i+1)
 
-transform :: (PrimMonad m, MVector v a)
-  => (MStream m a -> MStream m a) -> v (PrimState m) a -> m (v (PrimState m) a)
-{-# INLINE_STREAM transform #-}
+transform
+  :: (PrimMonad m, MVector v a)
+  => (Stream m a -> Stream m a) -> v (PrimState m) a -> m (v (PrimState m) a)
+{-# INLINE_FUSED transform #-}
 transform f v = fill v (f (mstream v))
 
-mstreamR :: (PrimMonad m, MVector v a) => v (PrimState m) a -> MStream m a
+mstreamR :: (PrimMonad m, MVector v a) => v (PrimState m) a -> Stream m a
 {-# INLINE mstreamR #-}
-mstreamR v = v `seq` n `seq` (MStream.unfoldrM get n `MStream.sized` Exact n)
+mstreamR v = v `seq` n `seq` (Stream.unfoldrM get n)
   where
     n = length v
 
@@ -276,10 +285,10 @@ mstreamR v = v `seq` n `seq` (MStream.unfoldrM get n `MStream.sized` Exact n)
         j = i-1
 
 fillR :: (PrimMonad m, MVector v a)
-           => v (PrimState m) a -> MStream m a -> m (v (PrimState m) a)
+      => v (PrimState m) a -> Stream m a -> m (v (PrimState m) a)
 {-# INLINE fillR #-}
 fillR v s = v `seq` do
-                      i <- MStream.foldM put n s
+                      i <- Stream.foldM put n s
                       return $ unsafeSlice i (n-i) v
   where
     n = length v
@@ -291,25 +300,28 @@ fillR v s = v `seq` do
       where
         j = i-1
 
-transformR :: (PrimMonad m, MVector v a)
-  => (MStream m a -> MStream m a) -> v (PrimState m) a -> m (v (PrimState m) a)
-{-# INLINE_STREAM transformR #-}
+transformR
+  :: (PrimMonad m, MVector v a)
+  => (Stream m a -> Stream m a) -> v (PrimState m) a -> m (v (PrimState m) a)
+{-# INLINE_FUSED transformR #-}
 transformR f v = fillR v (f (mstreamR v))
 
--- | Create a new mutable vector and fill it with elements from the 'Stream'.
--- The vector will grow exponentially if the maximum size of the 'Stream' is
+-- | Create a new mutable vector and fill it with elements from the 'Bundle'.
+-- The vector will grow exponentially if the maximum size of the 'Bundle' is
 -- unknown.
-unstream :: (PrimMonad m, MVector v a) => Stream a -> m (v (PrimState m) a)
--- NOTE: replace INLINE_STREAM by INLINE? (also in unstreamR)
-{-# INLINE_STREAM unstream #-}
-unstream s = munstream (Stream.liftStream s)
+unstream :: (PrimMonad m, MVector v a)
+         => Bundle u a -> m (v (PrimState m) a)
+-- NOTE: replace INLINE_FUSED by INLINE? (also in unstreamR)
+{-# INLINE_FUSED unstream #-}
+unstream s = munstream (Bundle.lift s)
 
 -- | Create a new mutable vector and fill it with elements from the monadic
 -- stream. The vector will grow exponentially if the maximum size of the stream
 -- is unknown.
-munstream :: (PrimMonad m, MVector v a) => MStream m a -> m (v (PrimState m) a)
-{-# INLINE_STREAM munstream #-}
-munstream s = case upperBound (MStream.size s) of
+munstream :: (PrimMonad m, MVector v a)
+          => MBundle m u a -> m (v (PrimState m) a)
+{-# INLINE_FUSED munstream #-}
+munstream s = case upperBound (MBundle.size s) of
                Just n  -> munstreamMax     s n
                Nothing -> munstreamUnknown s
 
@@ -319,12 +331,12 @@ munstream s = case upperBound (MStream.size s) of
 -- the shape of the vector) and one for when the vector has grown. To see the
 -- problem simply compile this:
 --
--- fromList = Data.Vector.Unboxed.unstream . Stream.fromList
+-- fromList = Data.Vector.Unboxed.unstream . Bundle.fromList
 --
 -- I'm not sure this still applies (19/04/2010)
 
-munstreamMax
-  :: (PrimMonad m, MVector v a) => MStream m a -> Int -> m (v (PrimState m) a)
+munstreamMax :: (PrimMonad m, MVector v a)
+             => MBundle m u a -> Int -> m (v (PrimState m) a)
 {-# INLINE munstreamMax #-}
 munstreamMax s n
   = do
@@ -334,17 +346,17 @@ munstreamMax s n
                        INTERNAL_CHECK(checkIndex) "munstreamMax" i n
                          $ unsafeWrite v i x
                        return (i+1)
-      n' <- MStream.foldM' put 0 s
+      n' <- MBundle.foldM' put 0 s
       return $ INTERNAL_CHECK(checkSlice) "munstreamMax" 0 n' n
              $ unsafeSlice 0 n' v
 
-munstreamUnknown
-  :: (PrimMonad m, MVector v a) => MStream m a -> m (v (PrimState m) a)
+munstreamUnknown :: (PrimMonad m, MVector v a)
+                 => MBundle m u a -> m (v (PrimState m) a)
 {-# INLINE munstreamUnknown #-}
 munstreamUnknown s
   = do
       v <- unsafeNew 0
-      (v', n) <- MStream.foldM put (v, 0) s
+      (v', n) <- MBundle.foldM put (v, 0) s
       return $ INTERNAL_CHECK(checkSlice) "munstreamUnknown" 0 n (length v')
              $ unsafeSlice 0 n v'
   where
@@ -353,25 +365,103 @@ munstreamUnknown s
                     v' <- unsafeAppend1 v i x
                     return (v',i+1)
 
--- | Create a new mutable vector and fill it with elements from the 'Stream'
+
+
+
+
+
+
+-- | Create a new mutable vector and fill it with elements from the 'Bundle'.
+-- The vector will grow exponentially if the maximum size of the 'Bundle' is
+-- unknown.
+vunstream :: (PrimMonad m, V.Vector v a)
+         => Bundle v a -> m (V.Mutable v (PrimState m) a)
+-- NOTE: replace INLINE_FUSED by INLINE? (also in unstreamR)
+{-# INLINE_FUSED vunstream #-}
+vunstream s = vmunstream (Bundle.lift s)
+
+-- | Create a new mutable vector and fill it with elements from the monadic
+-- stream. The vector will grow exponentially if the maximum size of the stream
+-- is unknown.
+vmunstream :: (PrimMonad m, V.Vector v a)
+           => MBundle m v a -> m (V.Mutable v (PrimState m) a)
+{-# INLINE_FUSED vmunstream #-}
+vmunstream s = case upperBound (MBundle.size s) of
+               Just n  -> vmunstreamMax     s n
+               Nothing -> vmunstreamUnknown s
+
+-- FIXME: I can't think of how to prevent GHC from floating out
+-- unstreamUnknown. That is bad because SpecConstr then generates two
+-- specialisations: one for when it is called from unstream (it doesn't know
+-- the shape of the vector) and one for when the vector has grown. To see the
+-- problem simply compile this:
+--
+-- fromList = Data.Vector.Unboxed.unstream . Bundle.fromList
+--
+-- I'm not sure this still applies (19/04/2010)
+
+vmunstreamMax :: (PrimMonad m, V.Vector v a)
+              => MBundle m v a -> Int -> m (V.Mutable v (PrimState m) a)
+{-# INLINE vmunstreamMax #-}
+vmunstreamMax s n
+  = do
+      v <- INTERNAL_CHECK(checkLength) "munstreamMax" n
+           $ unsafeNew n
+      let {-# INLINE_INNER copyChunk #-}
+          copyChunk i (Chunk m f) =
+            INTERNAL_CHECK(checkSlice) "munstreamMax.copyChunk" i m (length v) $ do
+              f (basicUnsafeSlice i m v)
+              return (i+m)
+
+      n' <- Stream.foldlM' copyChunk 0 (MBundle.chunks s)
+      return $ INTERNAL_CHECK(checkSlice) "munstreamMax" 0 n' n
+             $ unsafeSlice 0 n' v
+
+vmunstreamUnknown :: (PrimMonad m, V.Vector v a)
+                 => MBundle m v a -> m (V.Mutable v (PrimState m) a)
+{-# INLINE vmunstreamUnknown #-}
+vmunstreamUnknown s
+  = do
+      v <- unsafeNew 0
+      (v', n) <- Stream.foldlM copyChunk (v,0) (MBundle.chunks s)
+      return $ INTERNAL_CHECK(checkSlice) "munstreamUnknown" 0 n (length v')
+             $ unsafeSlice 0 n v'
+  where
+    {-# INLINE_INNER copyChunk #-}
+    copyChunk (v,i) (Chunk n f)
+      = do
+          let j = i+n
+          v' <- if basicLength v < j
+                  then unsafeGrow v (delay_inline max (enlarge_delta v) (j - basicLength v))
+                  else return v
+          INTERNAL_CHECK(checkSlice) "munstreamUnknown.copyChunk" i n (length v')
+            $ f (basicUnsafeSlice i n v')
+          return (v',j)
+
+
+
+
+-- | Create a new mutable vector and fill it with elements from the 'Bundle'
 -- from right to left. The vector will grow exponentially if the maximum size
--- of the 'Stream' is unknown.
-unstreamR :: (PrimMonad m, MVector v a) => Stream a -> m (v (PrimState m) a)
--- NOTE: replace INLINE_STREAM by INLINE? (also in unstream)
-{-# INLINE_STREAM unstreamR #-}
-unstreamR s = munstreamR (Stream.liftStream s)
+-- of the 'Bundle' is unknown.
+unstreamR :: (PrimMonad m, MVector v a)
+          => Bundle u a -> m (v (PrimState m) a)
+-- NOTE: replace INLINE_FUSED by INLINE? (also in unstream)
+{-# INLINE_FUSED unstreamR #-}
+unstreamR s = munstreamR (Bundle.lift s)
 
 -- | Create a new mutable vector and fill it with elements from the monadic
 -- stream from right to left. The vector will grow exponentially if the maximum
 -- size of the stream is unknown.
-munstreamR :: (PrimMonad m, MVector v a) => MStream m a -> m (v (PrimState m) a)
-{-# INLINE_STREAM munstreamR #-}
-munstreamR s = case upperBound (MStream.size s) of
+munstreamR :: (PrimMonad m, MVector v a)
+           => MBundle m u a -> m (v (PrimState m) a)
+{-# INLINE_FUSED munstreamR #-}
+munstreamR s = case upperBound (MBundle.size s) of
                Just n  -> munstreamRMax     s n
                Nothing -> munstreamRUnknown s
 
-munstreamRMax
-  :: (PrimMonad m, MVector v a) => MStream m a -> Int -> m (v (PrimState m) a)
+munstreamRMax :: (PrimMonad m, MVector v a)
+              => MBundle m u a -> Int -> m (v (PrimState m) a)
 {-# INLINE munstreamRMax #-}
 munstreamRMax s n
   = do
@@ -382,17 +472,17 @@ munstreamRMax s n
                       INTERNAL_CHECK(checkIndex) "munstreamRMax" i' n
                         $ unsafeWrite v i' x
                       return i'
-      i <- MStream.foldM' put n s
+      i <- MBundle.foldM' put n s
       return $ INTERNAL_CHECK(checkSlice) "munstreamRMax" i (n-i) n
              $ unsafeSlice i (n-i) v
 
-munstreamRUnknown
-  :: (PrimMonad m, MVector v a) => MStream m a -> m (v (PrimState m) a)
+munstreamRUnknown :: (PrimMonad m, MVector v a)
+                  => MBundle m u a -> m (v (PrimState m) a)
 {-# INLINE munstreamRUnknown #-}
 munstreamRUnknown s
   = do
       v <- unsafeNew 0
-      (v', i) <- MStream.foldM put (v, 0) s
+      (v', i) <- MBundle.foldM put (v, 0) s
       let n = length v'
       return $ INTERNAL_CHECK(checkSlice) "unstreamRUnknown" i (n-i) n
              $ unsafeSlice i (n-i) v'
@@ -480,7 +570,7 @@ unsafeDrop n v = unsafeSlice n (length v - n) v
 -- Overlapping
 -- -----------
 
--- Check whether two vectors overlap.
+-- | Check whether two vectors overlap.
 overlaps :: MVector v a => v s a -> v s a -> Bool
 {-# INLINE overlaps #-}
 overlaps = basicOverlaps
@@ -492,7 +582,7 @@ overlaps = basicOverlaps
 new :: (PrimMonad m, MVector v a) => Int -> m (v (PrimState m) a)
 {-# INLINE new #-}
 new n = BOUNDS_CHECK(checkLength) "new" n
-      $ unsafeNew n
+      $ unsafeNew n >>= \v -> basicInitialize v >> return v
 
 -- | Create a mutable vector of the given length. The length is not checked.
 unsafeNew :: (PrimMonad m, MVector v a) => Int -> m (v (PrimState m) a)
@@ -510,7 +600,7 @@ replicate n x = basicUnsafeReplicate (delay_inline max 0 n) x
 -- and fill it with values produced by repeatedly executing the monadic action.
 replicateM :: (PrimMonad m, MVector v a) => Int -> m a -> m (v (PrimState m) a)
 {-# INLINE replicateM #-}
-replicateM n m = munstream (MStream.replicateM n m)
+replicateM n m = munstream (MBundle.replicateM n m)
 
 -- | Create a copy of a mutable vector.
 clone :: (PrimMonad m, MVector v a) => v (PrimState m) a -> m (v (PrimState m) a)
@@ -529,27 +619,37 @@ grow :: (PrimMonad m, MVector v a)
                 => v (PrimState m) a -> Int -> m (v (PrimState m) a)
 {-# INLINE grow #-}
 grow v by = BOUNDS_CHECK(checkLength) "grow" by
-          $ unsafeGrow v by
+          $ do vnew <- unsafeGrow v by
+               basicInitialize $ basicUnsafeSlice (length v) by vnew
+               return vnew
 
 growFront :: (PrimMonad m, MVector v a)
                 => v (PrimState m) a -> Int -> m (v (PrimState m) a)
 {-# INLINE growFront #-}
 growFront v by = BOUNDS_CHECK(checkLength) "growFront" by
-               $ unsafeGrowFront v by
+               $ do vnew <- unsafeGrowFront v by
+                    basicInitialize $ basicUnsafeSlice 0 by vnew
+                    return vnew
 
+enlarge_delta :: MVector v a => v s a -> Int
 enlarge_delta v = max (length v) 1
 
 -- | Grow a vector logarithmically
 enlarge :: (PrimMonad m, MVector v a)
                 => v (PrimState m) a -> m (v (PrimState m) a)
 {-# INLINE enlarge #-}
-enlarge v = unsafeGrow v (enlarge_delta v)
+enlarge v = do vnew <- unsafeGrow v by
+               basicInitialize $ basicUnsafeSlice (length v) by vnew
+               return vnew
+  where
+    by = enlarge_delta v
 
 enlargeFront :: (PrimMonad m, MVector v a)
                 => v (PrimState m) a -> m (v (PrimState m) a, Int)
 {-# INLINE enlargeFront #-}
 enlargeFront v = do
                    v' <- unsafeGrowFront v by
+                   basicInitialize $ basicUnsafeSlice 0 by v'
                    return (v', by)
   where
     by = enlarge_delta v
@@ -576,7 +676,7 @@ unsafeGrowFront v by = UNSAFE_CHECK(checkLength) "unsafeGrowFront" by
 -- ------------------------
 
 -- | Reset all elements of the vector to some undefined value, clearing all
--- references to external objects. This is usually a noop for unboxed vectors. 
+-- references to external objects. This is usually a noop for unboxed vectors.
 clear :: (PrimMonad m, MVector v a) => v (PrimState m) a -> m ()
 {-# INLINE clear #-}
 clear = basicClear
@@ -596,6 +696,12 @@ write :: (PrimMonad m, MVector v a) => v (PrimState m) a -> Int -> a -> m ()
 write v i x = BOUNDS_CHECK(checkIndex) "write" i (length v)
             $ unsafeWrite v i x
 
+-- | Modify the element at the given position.
+modify :: (PrimMonad m, MVector v a) => v (PrimState m) a -> (a -> a) -> Int -> m ()
+{-# INLINE modify #-}
+modify v f i = BOUNDS_CHECK(checkIndex) "modify" i (length v)
+             $ unsafeModify v f i
+
 -- | Swap the elements at the given positions.
 swap :: (PrimMonad m, MVector v a) => v (PrimState m) a -> Int -> Int -> m ()
 {-# INLINE swap #-}
@@ -622,6 +728,13 @@ unsafeWrite :: (PrimMonad m, MVector v a)
 unsafeWrite v i x = UNSAFE_CHECK(checkIndex) "unsafeWrite" i (length v)
                   $ basicUnsafeWrite v i x
 
+-- | Modify the element at the given position. No bounds checks are performed.
+unsafeModify :: (PrimMonad m, MVector v a) => v (PrimState m) a -> (a -> a) -> Int -> m ()
+{-# INLINE unsafeModify #-}
+unsafeModify v f i = UNSAFE_CHECK(checkIndex) "unsafeModify" i (length v)
+                   $ basicUnsafeRead v i >>= \x ->
+                     basicUnsafeWrite v i (f x)
+
 -- | Swap the elements at the given positions. No bounds checks are performed.
 unsafeSwap :: (PrimMonad m, MVector v a)
                 => v (PrimState m) a -> Int -> Int -> m ()
@@ -666,7 +779,7 @@ copy dst src = BOUNDS_CHECK(check) "copy" "overlapping vectors"
 
 -- | Move the contents of a vector. The two vectors must have the same
 -- length.
--- 
+--
 -- If the vectors do not overlap, then this is equivalent to 'copy'.
 -- Otherwise, the copying is performed as if the source vector were
 -- copied to a temporary vector and then the temporary vector was copied
@@ -692,7 +805,7 @@ unsafeCopy dst src = UNSAFE_CHECK(check) "unsafeCopy" "length mismatch"
 
 -- | Move the contents of a vector. The two vectors must have the same
 -- length, but this is not checked.
--- 
+--
 -- If the vectors do not overlap, then this is equivalent to 'unsafeCopy'.
 -- Otherwise, the copying is performed as if the source vector were
 -- copied to a temporary vector and then the temporary vector was copied
@@ -709,9 +822,9 @@ unsafeMove dst src = UNSAFE_CHECK(check) "unsafeMove" "length mismatch"
 -- ------------
 
 accum :: (PrimMonad m, MVector v a)
-        => (a -> b -> a) -> v (PrimState m) a -> Stream (Int, b) -> m ()
+      => (a -> b -> a) -> v (PrimState m) a -> Bundle u (Int, b) -> m ()
 {-# INLINE accum #-}
-accum f !v s = Stream.mapM_ upd s
+accum f !v s = Bundle.mapM_ upd s
   where
     {-# INLINE_INNER upd #-}
     upd (i,b) = do
@@ -722,9 +835,9 @@ accum f !v s = Stream.mapM_ upd s
     !n = length v
 
 update :: (PrimMonad m, MVector v a)
-                        => v (PrimState m) a -> Stream (Int, a) -> m ()
+                        => v (PrimState m) a -> Bundle u (Int, a) -> m ()
 {-# INLINE update #-}
-update !v s = Stream.mapM_ upd s
+update !v s = Bundle.mapM_ upd s
   where
     {-# INLINE_INNER upd #-}
     upd (i,b) = BOUNDS_CHECK(checkIndex) "update" i n
@@ -733,9 +846,9 @@ update !v s = Stream.mapM_ upd s
     !n = length v
 
 unsafeAccum :: (PrimMonad m, MVector v a)
-            => (a -> b -> a) -> v (PrimState m) a -> Stream (Int, b) -> m ()
+            => (a -> b -> a) -> v (PrimState m) a -> Bundle u (Int, b) -> m ()
 {-# INLINE unsafeAccum #-}
-unsafeAccum f !v s = Stream.mapM_ upd s
+unsafeAccum f !v s = Bundle.mapM_ upd s
   where
     {-# INLINE_INNER upd #-}
     upd (i,b) = do
@@ -746,9 +859,9 @@ unsafeAccum f !v s = Stream.mapM_ upd s
     !n = length v
 
 unsafeUpdate :: (PrimMonad m, MVector v a)
-                        => v (PrimState m) a -> Stream (Int, a) -> m ()
+                        => v (PrimState m) a -> Bundle u (Int, a) -> m ()
 {-# INLINE unsafeUpdate #-}
-unsafeUpdate !v s = Stream.mapM_ upd s
+unsafeUpdate !v s = Bundle.mapM_ upd s
   where
     {-# INLINE_INNER upd #-}
     upd (i,b) = UNSAFE_CHECK(checkIndex) "accum" i n
@@ -794,16 +907,16 @@ unstablePartition f !v = from_left 0 (length v)
                                from_left (i+1) j
                         else from_right i (j-1)
 
-unstablePartitionStream :: (PrimMonad m, MVector v a)
-        => (a -> Bool) -> Stream a -> m (v (PrimState m) a, v (PrimState m) a)
-{-# INLINE unstablePartitionStream #-}
-unstablePartitionStream f s
-  = case upperBound (Stream.size s) of
+unstablePartitionBundle :: (PrimMonad m, MVector v a)
+        => (a -> Bool) -> Bundle u a -> m (v (PrimState m) a, v (PrimState m) a)
+{-# INLINE unstablePartitionBundle #-}
+unstablePartitionBundle f s
+  = case upperBound (Bundle.size s) of
       Just n  -> unstablePartitionMax f s n
       Nothing -> partitionUnknown f s
 
 unstablePartitionMax :: (PrimMonad m, MVector v a)
-        => (a -> Bool) -> Stream a -> Int
+        => (a -> Bool) -> Bundle u a -> Int
         -> m (v (PrimState m) a, v (PrimState m) a)
 {-# INLINE unstablePartitionMax #-}
 unstablePartitionMax f s n
@@ -818,20 +931,20 @@ unstablePartitionMax f s n
             | otherwise = do
                             unsafeWrite v (j-1) x
                             return (i, j-1)
-                                
-      (i,j) <- Stream.foldM' put (0, n) s
+
+      (i,j) <- Bundle.foldM' put (0, n) s
       return (unsafeSlice 0 i v, unsafeSlice j (n-j) v)
 
-partitionStream :: (PrimMonad m, MVector v a)
-        => (a -> Bool) -> Stream a -> m (v (PrimState m) a, v (PrimState m) a)
-{-# INLINE partitionStream #-}
-partitionStream f s
-  = case upperBound (Stream.size s) of
+partitionBundle :: (PrimMonad m, MVector v a)
+        => (a -> Bool) -> Bundle u a -> m (v (PrimState m) a, v (PrimState m) a)
+{-# INLINE partitionBundle #-}
+partitionBundle f s
+  = case upperBound (Bundle.size s) of
       Just n  -> partitionMax f s n
       Nothing -> partitionUnknown f s
 
 partitionMax :: (PrimMonad m, MVector v a)
-  => (a -> Bool) -> Stream a -> Int -> m (v (PrimState m) a, v (PrimState m) a)
+  => (a -> Bool) -> Bundle u a -> Int -> m (v (PrimState m) a, v (PrimState m) a)
 {-# INLINE partitionMax #-}
 partitionMax f s n
   = do
@@ -847,9 +960,9 @@ partitionMax f s n
             | otherwise = let j' = j-1 in
                           do
                             unsafeWrite v j' x
-                            return (i,j') 
-                            
-      (i,j) <- Stream.foldM' put (0,n) s
+                            return (i,j')
+
+      (i,j) <- Bundle.foldM' put (0,n) s
       INTERNAL_CHECK(check) "partitionMax" "invalid indices" (i <= j)
         $ return ()
       let l = unsafeSlice 0 i v
@@ -858,13 +971,13 @@ partitionMax f s n
       return (l,r)
 
 partitionUnknown :: (PrimMonad m, MVector v a)
-        => (a -> Bool) -> Stream a -> m (v (PrimState m) a, v (PrimState m) a)
+        => (a -> Bool) -> Bundle u a -> m (v (PrimState m) a, v (PrimState m) a)
 {-# INLINE partitionUnknown #-}
 partitionUnknown f s
   = do
       v1 <- unsafeNew 0
       v2 <- unsafeNew 0
-      (v1', n1, v2', n2) <- Stream.foldM' put (v1, 0, v2, 0) s
+      (v1', n1, v2', n2) <- Bundle.foldM' put (v1, 0, v2, 0) s
       INTERNAL_CHECK(checkSlice) "partitionUnknown" 0 n1 (length v1')
         $ INTERNAL_CHECK(checkSlice) "partitionUnknown" 0 n2 (length v2')
         $ return (unsafeSlice 0 n1 v1', unsafeSlice 0 n2 v2')
