@@ -1,12 +1,12 @@
 module Main (main) where
 
 import qualified Tests.Vector
-import qualified Tests.Stream
+import qualified Tests.Bundle
 import qualified Tests.Move
 
 import Test.Framework (defaultMain)
 
-main = defaultMain $ Tests.Stream.tests
+main = defaultMain $ Tests.Bundle.tests
                   ++ Tests.Vector.tests
                   ++ Tests.Move.tests
 
